function dia_Ts
% DIA_TS Erzeugt ein T,s-Diagramm
% DIA_TS()
%
% Eigentlich ist dies ein Matlab-Script und keine Funktion,
% damit es in eine Datei passt ist es allerdings als eine
% solche geschrieben.
%
% Parameter zur Anzeige koennen hier zu Beginn eingestellt werden.
% Dieses Skript kuemmert sich nur um die Diagramme und nutzt
% XSteam von Magnus Holmgren.
%
% V. Hartung 2019 https://hartrusion.com

% Aktuelles Figure-Fenster leeren und Achsensystem erstellen
clf;
h_ax = axes;
hold(h_ax,'on');

% Vektor mit durchzurechnenden Entropiewerten definieren (X-Achse)
s = 0:0.2:9; % Feine Darstellung: 0:0.01:9
% Darzustellende Dampfanteilslinien
x = 0.1:0.1:0.9;
% Darzustellende Isochore
v_iso = [0.01,0.03,0.1,0.3,1,3,10,30,100];
% Darzustellende Isobaren
p_iso = [0.01,0.1,1,4,10,25,50,100,150,250,500,1000];

% Parameter fuer die Rechnung (reduzieren, um Geschw. zu erhoehen)
n_x = 10; % Anzahl Druckwerte zur Berechnung (Highres: 300)
n_p_iso = 300; % Anzahl Druckwerte zur Berechnung Isobare (Highres: 1500)
n_sat = 50; % Anzahl Druckwerte zur Glockenkurve (Highres: 300)

%% Isochoren erzeugen und beschriften
T_v_txt = -100 * s + 1200; %Geradengleichung fuer Textpositionierung
p_v = logspace(-2,log10(500),n_p_iso); % Druck-Vektor erzeugen
% Eine Volumen-Matrix mit Volumen von s und p erzeugen, v_M = f(s,p)
v_M = zeros(length(s),length(p_v)); % Initialisierung
for idx_n=1:length(s)
    for idx_m=1:length(p_v)
        v_M(idx_n,idx_m)=XSteam('v_ps',p_v(idx_m),s(idx_n));
    end
end
T_v = zeros(1,length(s)); % Initialisierung
for idx_n=1:length(v_iso) % Fuer die vorgegebenen Isobaren    
    for idx_m=1:length(s) % Zeilenweise (s steht in Zeile) durchgehen:
        % Ermitteln, welcher Druck der gesuchten Isobaren am nachsten ist
        pn = inversemin(p_v,v_M(idx_m,:),v_iso(idx_n));
        T_v(idx_m) = XSteam('T_ps',pn,s(idx_m)) + 273; %Temperatur dazu
        if T_v(idx_m) > 1050    % Begrenzung bis 1020 K, sonst gibts Chaos.
            T_v(idx_m) = NaN;
        end
    end
    plot(s,T_v,'Color',[0,0.4,0.4])
    
    [txt_x,txt_y] = intersmin(s,T_v,T_v_txt);
    text(txt_x*0.99,txt_y,[num2str(v_iso(idx_n)),' m3/kg'],...
        'FontSize',6,'rotation',70,'Color',[0,0.4,0.4]);
end

%% Isobare erzeugen und beschriften
% Geradengleichung festlegen (Vektor T(s)) fuer die Beschriftung
T_piso_txt = -40 * s + 950;
T_iso = zeros(1,length(s)); % Initialisierung
for idx_n=1:length(p_iso) % Fuer jeden Wert aus dem Isobaren-Vektor
    % Zugehoerige Temperatur errechnen ueber T=f(p,s)
    for idx_m = 1:length(s)
        T_iso(idx_m)=XSteam('T_ps',p_iso(idx_n),s(idx_m))+273;
    end
    plot(s,T_iso,'--','Color',[0.4 0.4 0.4]) % Linie einzeichnen
    % Isobaren-Beschriftung unter der Glockenkurve
    txt_x = 4; % X-Position als Entropieangabe
    txt_y = XSteam('T_ps',p_iso(idx_n),txt_x)+273 + 10;
    % Wenn der y-Wert unterhalb der Glockenkurve ist, beschriften:
    if (XSteam('x_ps',p_iso(idx_n),txt_x) < 1)
        text(txt_x,txt_y,[num2str(p_iso(idx_n)), ' bar'],...
            'FontSize',8,'Color',[0.4 0.4 0.4]);
    end
    % Isobaren-Beschriftung bei ueberhitztem Dampf einfuegen:
    % Text liegt auf der oben definierten Gerade
    [txt_x,txt_y] = intersmin(s,T_piso_txt,T_iso);
    if txt_x < 9 %Nur bis s=9, da sonst y nicht richtig definiert ist.
        text(txt_x*0.98,txt_y,[num2str(p_iso(idx_n)), ' bar'],...
            'FontSize',6,'rotation',65,'Color',[0.4 0.4 0.4])
    end
end

%% Dampfanteilslinien erzeugen
p_x = logspace(-2,log10(220),n_x); % Druck-Vektor erzeugen
% Fuer jede Dampfanteilslinie jeden Druckvektor durcharbeiten. Es wird zu
% den Druecken p_x jeweils T und s errechnet, dies wird dann gezeichnet.
h_x = zeros(size(p_x));
s_x = zeros(size(p_x));
T_x = zeros(size(p_x));
for idx_n=1:length(x) % Fuer jede X-Linie wird der Druck-Vektor durch-
    for idx_m=1:length(p_x) % gerechnet und T-s-Paare erzeugt.
        h_x(idx_m)=XSteam('h_px',p_x(idx_m),x(idx_n));
        s_x(idx_m)=XSteam('s_ph',p_x(idx_m),h_x(idx_m));
        T_x(idx_m)=XSteam('T_ps',p_x(idx_m),s_x(idx_m))+273;
    end
    plot(s_x,T_x,'r');
    txt_y=265;
    txt_x=s_x(1)-0.3;
    text(txt_x,txt_y,['  ',num2str(x(idx_n)*100), ' %'],...
        'FontSize',8,'Color',[0.7,0,0]);
end

%% Glockenkurve zeichnen
p_sat = [logspace(-2,log10(220),n_sat),221.2]; % Druckvektor bis krit. Punkt
c = 0; % Zaehlvariable
T_sat = zeros(1,length(p_sat)-1); % Initialisierung
for idx_n=1:(2*length(p_sat)-1) % Druck-Vektor wird 2 Mal durchgerechnet:
    if ( idx_n < length(p_sat) ) % Teil 1: Siedelinie
        T_sat(idx_n)=XSteam('Tsat_p',p_sat(idx_n))+273; % Sattdampft.
    elseif ( idx_n == length(p_sat) ) % Kritischer Punkt erreicht
        T_sat(idx_n)=374.14+273;  % Sattdampftemp. im kritischen Punkt
    else % Teil 2: Taulinie
        c = c+2; % Um p_sat rueckwaerts abzuarbeiten.
        T_sat(idx_n)=XSteam('Tsat_p',p_sat(idx_n-c))+273;
    end
end
% Zu den errechneten Temperaturen die Entropie berechnen:
s_sat = zeros(1,length(T_sat)); % Initialisierung
for idx_n=1:(length(T_sat))
    if ( idx_n < length(p_sat) ) % Teil 1: Siedelinie
        s_sat(idx_n)=XSteam('sL_T',(T_sat(idx_n)-273));
    elseif ( idx_n == length(p_sat) ) % Kritischer Punkt erreicht
        s_sat(idx_n)=4.4298; % Entropie im kritischen Punkt
    else % Teil 2: Taulinie
        s_sat(idx_n)=XSteam('sV_T',(T_sat(idx_n)-273));
    end
end
plot(s_sat,T_sat,'r','LineWidth',1);
% Text manuell einfuegen
text(2.1,250+273,'x=0','Color',[0.7,0,0]);
text(6.2,250+273,'x=1','Color',[0.7,0,0]);

%% Kritischen Punkt einzeichnen
plot(4.4298,374.14+273,'*','MarkerSize',4,'Color',[0,0,1]);

%% Diagramm-Einstellungen
grid on;
axis([0, 9, 200, 900]);
xlabel('Entropie s in kJ/kg/K');%,'FontSize',12);
ylabel('Temperatur T in K');%,'FontSize',12);
set(h_ax,'GridAlpha',1,...
    'GridColor',[0.8,0.8,0.8]);
    
end % function

%% Lokale Funktionen
function varargout = inversemin(x,y,y0)
% INVERSEMIN Ermittelt, wo y0 am nahesten bei Gerade (x,y) liegt
% [X0,POS] = INVERSEMIN(X,Y,Y0)
[~,pos] = min(abs(y-y0));
varargout{1} = x(pos);
varargout{2} = pos;
end

function varargout = intersmin(x,y1,y2)
% [x0,y0,pos] = INTERSMIN(X,Y1,Y2)
% Position x und y des kleinsten Abstands zwischen zwei Geraden x,y1 und
% x,y2 finden, gibt x als Wert zurueck sowie die Position.
[~,pos] = min(abs(y2-y1)); % Position des kleinsten Abstandes
varargout{1} = x(pos);
varargout{2} = (y1(pos)+y2(pos))./2; % Mittelwert
varargout{3} = pos;
end