function dia_hs
% DIA_HS Erzeugt ein h,s-Diagramm
% DIA_HS()
%
% Eigentlich ist dies ein Matlab-Script und keine Funktion,
% damit es in eine Datei passt ist es allerdings als eine
% solche geschrieben.
%
% Parameter zur Anzeige koennen hier zu Beginn eingestellt werden.
% Dieses Skript kuemmert sich nur um die Diagramme und nutzt
% XSteam von Magnus Holmgren.
%
% V. Hartung 2019 https://hartrusion.com

clf;
h_ax = axes;
hold(h_ax,'on');

% Einfaches Diagramm
% Achsenskalierung/Diagrammbereich festlegen (wg. Beschriftung)
s_min = 0;
s_stp = 1;
s_max = 12;
h_min = 0;
h_stp = 500;
h_max = 4000;
% Vektor mit durchzurechnenden Entropiewerten definieren (X-Achse)
s = s_min:0.1:s_max;
% Darzustellende Isochore
v_iso = [0.01,0.03,0.1,0.3,1,3,10,30,100];
% Darzustellende Isobare
p_iso = [0.01,0.025,0.05,0.1,0.25,0.5,1,2,5,10,20,50,100,200,500,1000];
% Darzustellende Isotherme
T_iso = [50,100:100:700];
% Darzustellende Dampfanteilslinien
x = 0.1:0.1:1;

% Parameter fuer die Rechnung (reduzieren, um Geschw. zu erhoehen)
n_p_isov = 100; % Anzahl Druckwerte fuer Isochore (Highres: 1000)
n_p_sat = 200; % Anzahl Druckwerte fuer Glockenkurve (Highres: 2000)
n_p_isot = 20; % Anzahl Druckwerte fuer Isotherme (Highres: 200)
n_p_x = 100; % Anzahl Druckwerte fuer Dampfanteilslinien (Highres: 1000)

% Detaillierteres Diagramm, fuer groessere Darstellung
% % Achsenskalierung/Diagrammbereich festlegen (wg. Beschriftung)
% s_min = 3;
% s_stp = 0.5;
% s_max = 12;
% h_min = 1800;
% h_stp = 100;
% h_max = 4000;
% % Vektor mit durchzurechnenden Entropiewerten definieren (X-Achse)
% s = s_min:0.1:s_max;
% % Darzustellende Isochore
% v_iso = [0.01,0.015,0.03,0.05,0.1,0.15,0.3,0.5,1,1.5,3,5,10,15,30,50,100,150,300,500];
% % Darzustellende Isobare
% p_iso = [0.01,0.015,0.03,0.05,0.1,0.15,0.3,0.5,1,1.5,3,5,10,15,30,50,100,150,300,500,1000,1500];
% % Darzustellende Isotherme
% T_iso = [20,50:50:750];
% % Darzustellende Dampfanteilslinien
% x = 0.1:0.025:1;

% Detaillierteres Diagramm im ueberhitzten bereich
% % Achsenskalierung/Diagrammbereich festlegen (wg. Beschriftung)
% s_min = 5;
% s_stp = 0.5;
% s_max = 12;
% h_min = 2400;
% h_stp = 100;
% h_max = 4000;
% % Vektor mit durchzurechnenden Entropiewerten definieren (X-Achse)
% s = s_min:0.1:s_max;
% % Darzustellende Isochore
% v_iso = [0.01,0.015,0.03,0.05,0.1,0.15,0.3,0.5,1,1.5,3,5,10,15,30,50,100,150,300,500];
% % Darzustellende Isobare
% p_iso = [0.01,0.015,0.03,0.05,0.1,0.15,0.3,0.5,1,1.5,3,5,10,15,30,50,100,150,300,500,1000,1500];
% % Darzustellende Isotherme
% T_iso = [20,50:50:750];
% % Darzustellende Dampfanteilslinien
% x = 0.1:0.025:1;

% Kleiner Ausschnitt unten
% % Achsenskalierung/Diagrammbereich festlegen (wg. Beschriftung)
% s_min = 0;
% s_stp = 0.5;
% s_max = 5;
% h_min = 0;
% h_stp = 200;
% h_max = 1400;
% % Vektor mit durchzurechnenden Entropiewerten definieren (X-Achse)
% s = s_min:0.1:s_max;
% % Darzustellende Isochore
% v_iso = [0.01,0.03,0.1,0.3,1,3,10,30,100];
% % Darzustellende Isobare
% p_iso = [0.01,0.025,0.05,0.1,0.25,0.5,1,2,5,10,20,50,100,200,500,1000];
% % Darzustellende Isotherme
% T_iso = [50,100:100:700];
% % Darzustellende Dampfanteilslinien
% x = 0.1:0.05:0.5;

%% Isochoren erzeugen
p_v = logspace(-2,log10(500),n_p_isov); % Druck-Vektor erzeugen
% Eine Volumen-Matrix mit Volumen von s und p erzeugen, v_M = f(s,p)
v_M = zeros(length(s),length(p_v)); % Initialisierung
for idx_n=1:length(s)
    for idx_m=1:length(p_v)
        v_M(idx_n,idx_m)=XSteam('v_ps',p_v(idx_m),s(idx_n));
    end
end
h_v = zeros(1,length(s)); % Initialisierung
for idx_n=1:length(v_iso) % Fuer die vorgegebenen Isobaren    
    for idx_m=1:length(s) % Zeilenweise (s steht in Zeile) durchgehen:
        % Ermitteln, welcher Druck der gesuchten Isobaren am nachsten ist
        pn = inversemin(p_v,v_M(idx_m,:),v_iso(idx_n));
        h_v(idx_m) = XSteam('h_ps',pn,s(idx_m)); % Enthalpie dazu
        if h_v(idx_m) > 3900 % Begrenzung bis 3900 kJ/kg, sonst gibts Chaos
            h_v(idx_m) = NaN;
        end
    end
    plot(s,h_v,'Color',[0,0.4,0.4]);
    txt_y = 2800;
    txt_x = inversemin(s,h_v,txt_y);
    text(txt_x,txt_y,[num2str(v_iso(idx_n)),' m3/kg'],...
        'FontSize',6,'rotation',65,'Color',[0,0.4,0.4]);
end

%% Isobare erzeugen
h_iso=zeros(size(s)); % Initialisierung
for idx_n=1:length(p_iso)
    %Isobare jeweils ermitteln und in Vektor T_iso schreiben
    for idx_m = 1:length(s)
        h_iso(idx_m)=XSteam('h_ps',p_iso(idx_n),s(idx_m));
    end
    plot(s,h_iso,'--','Color',[0.4 0.4 0.4]);
    txt_y=min([h_max*0.85,max(h_iso)*0.85]);
    txt_x=XSteam('s_ph',p_iso(idx_n),txt_y)-0.1;
    text(txt_x,txt_y,[num2str(p_iso(idx_n)), ' bar'],...
        'FontSize',6,'rotation',70,'Color',[0.4,0.4,0.4])
end

%% Isotherme erzeugen
p_isotherm = logspace(-2,3,n_p_isot);
s_iso=zeros(1,length(p_isotherm));
h_iso=zeros(1,length(p_isotherm));
for idx_n = 1:length(T_iso) % Fuer jede zu zeichnende Isotherme
    for idx_m = 1:length(p_isotherm) % Ueber den Druck durchrechnen
        s_iso(idx_m)=XSteam('s_pT',p_isotherm(idx_m),T_iso(idx_n));
        % Handelt es sich um das Sattdampfgebiet?
        if (XSteam('vx_ps',p_isotherm(idx_m),s_iso(idx_m)) < 1)
            h_iso(idx_m)=NaN; % Keine Punkte zeichnen
        else
            h_iso(idx_m)=XSteam('h_ps',p_isotherm(idx_m),s_iso(idx_m));
        end
    end
    plot(s_iso,h_iso,'Color',[0,0,1]);
    text(s_iso(1),h_iso(1),['  ',num2str(T_iso(idx_n)), ' °C'],...
        'FontSize',8,'Color',[0,0,0.7]);
end

%% Glockenkurve zeichnen
p_sat = [logspace(-2,log10(220),n_p_sat),221.2]; % Druckvektor bis k. Punkt
c = 0; % Zaehlvariable
T_sat = zeros(1,length(p_sat)-1); % Initialisierung
for idx_n=1:(2*length(p_sat)-1) % Druck-Vektor wird 2 Mal durchgerechnet:
    if ( idx_n < length(p_sat) ) % Teil 1: Siedelinie
        T_sat(idx_n)=XSteam('Tsat_p',p_sat(idx_n)); % Sattdampft.
    elseif ( idx_n == length(p_sat) ) % Kritischer Punkt erreicht
        T_sat(idx_n)=374.14;  % Sattdampftemp. im kritischen Punkt
    else % Teil 2: Taulinie
        c = c+2; % Um p_sat rueckwaerts abzuarbeiten.
        T_sat(idx_n)=XSteam('Tsat_p',p_sat(idx_n-c));
    end
end
% Zu den errechneten Temperaturen die Entropie berechnen:
s_sat=zeros(1,length(T_sat));
for idx_n=1:(length(T_sat)) % Teil 1: Siedelinie
    if ( idx_n < length(p_sat) )
        s_sat(idx_n)=XSteam('sL_T',T_sat(idx_n));
    elseif ( idx_n == length(p_sat) ) % Kritischer Punkt erreicht
        s_sat(idx_n)=4.4298;  % Entropie im kritischen Punkt
    else % Teil 2: Taulinie
        s_sat(idx_n)=XSteam('sV_T',T_sat(idx_n));
    end
end
% Aus Temperaturen die Enthalpie berechnen
h_sat = zeros(1,length(T_sat)); % Initialisierung
for idx_n=1:(length(T_sat))
    if ( idx_n < length(p_sat) )
        h_sat(idx_n)=XSteam('hL_T',T_sat(idx_n));
    elseif ( idx_n == length(p_sat) ) % Kritischer Punkt
        h_sat(idx_n)=2107.4;
    else
        h_sat(idx_n)=XSteam('hV_T',T_sat(idx_n));
    end
end
plot(s_sat,h_sat,'r','LineWidth',2);
%text(2.1,250,'x=0');
%text(6.2,250,'x=1');

%% Dampfanteilslinien erzeugen
p_x = logspace(-2,3,n_p_x); % Druckvektor zum durchrechnen
h_x = zeros(1,length(p_x)); % Init
s_x = zeros(1,length(p_x)); % Init
for idx_n=1:length(x)
    for idx_m=1:length(p_x)
        h_x(idx_m)=XSteam('h_px',p_x(idx_m),x(idx_n));
        s_x(idx_m)=XSteam('s_ph',p_x(idx_m),h_x(idx_m));
    end
    plot(s_x,h_x,'r','LineWidth',1);
    txt_y=XSteam('h_px',min(p_x),x(idx_n));
    txt_x=XSteam('s_ph',min(p_x),txt_y);
    if (txt_y >= h_min && txt_y <= h_max)
        text(txt_x,txt_y,['  ',num2str(x(idx_n)*100), ' %'],...
            'FontSize',8,'Color',[0.7,0,0])
    end
end

%% Diagrammeinstellungen
grid on;
axis([s_min, s_max, h_min, h_max]);
xlabel('Entropie s in kJ/kg/K');
ylabel('Enthalpie h in kJ/kg');

set(h_ax,'XTick',s_min:s_stp:s_max,...
    'YTick',h_min:h_stp:h_max)

end % function

%% Lokale Funktionen
function varargout = inversemin(x,y,y0)
% INVERSEMIN Ermittelt, wo y0 am nahesten bei Gerade (x,y) liegt
% [X0,POS] = INVERSEMIN(X,Y,Y0)
[~,pos] = min(abs(y-y0));
varargout{1} = x(pos);
varargout{2} = pos;
end